import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.decomposition import PCA
from nltk.tokenize import word_tokenize
from nltk.corpus import stopwords
from nltk import download
import numpy as np
from sklearn.metrics.pairwise import cosine_similarity
from nltk.tokenize.casual import casual_tokenize

# Télécharger les données nécessaires de NLTK
download('punkt')
download('stopwords')

# Lire les données
pd.options.display.width = 120
sms = pd.read_csv('wiki.csv')

# Fonction de tokenization personnalisée pour ignorer les ponctuations
def custom_tokenize(text):
    tokens = word_tokenize(text)
    return [word.lower() for word in tokens if word.isalnum()]

# Créer le vecteur TF-IDF avec tokenization personnalisée
tfidf = TfidfVectorizer(tokenizer=custom_tokenize, stop_words='english')
tfidf_docs = tfidf.fit_transform(raw_documents=sms.Summary).toarray()
terms = tfidf.get_feature_names_out()

# Appliquer la transformation PCA
n_components = 20 
pca = PCA(n_components=n_components)
pca_tfidf_docs = pca.fit_transform(tfidf_docs)

# Obtenir les scores de chargement
loading_scores = pca.components_

# Fonction pour afficher les mots principaux pour chaque composante principale
def display_topics(components, feature_names, n_top_words = 100):
    for i, component in enumerate(components):
        top_indices = component.argsort()[-n_top_words:][::-1]
        top_features = [feature_names[i] for i in top_indices]
        top_scores = component[top_indices]
        print(f"Principal Component {i+1}:")
        for feature, score in zip(top_features, top_scores):
            print(f"  {feature}: {score:.4f}")
        print()

# Afficher les mots principaux pour chaque composante principale
#display_topics(loading_scores, terms)

while True:
    query = input("search: ")
    if(query == "stop"):
        break
    tokens = word_tokenize(query.lower())
    tokens = [word for word in tokens if word.isalnum() and word not in stopwords.words('english')]
    processed_query = ' '.join(tokens)
    query_vector = tfidf.transform([processed_query]).toarray()

    # Apply PCA transformation to the query vector
    pca_query_vector = pca.transform(query_vector)
    #print(pca_query_vector)

    # Calculate cosine similarities with PCA-transformed vectors
    cosine_similarities = cosine_similarity(pca_query_vector, pca_tfidf_docs).flatten()
    top_indices = cosine_similarities.argsort()[-1:][::-1]
    res = sms.iloc[top_indices]

    # Debug: Print top similarities and corresponding indices
    #print(f"Top cosine similarities: {cosine_similarities[top_indices]}")
    #print(f"Top indices: {top_indices}")

    for _, row in res.iterrows():
        print(row['URL'])
