import pandas as pd
import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity
import nltk
from nltk.corpus import stopwords
import string

nltk.download('stopwords')

def preprocess_text(text):
    text = text.lower()
    # Remove punctuation
    text = text.translate(str.maketrans('', '', string.punctuation))
    tokens = text.split()
    # Remove stop words
    tokens = [word for word in tokens if word not in stopwords.words('english')]
    return ' '.join(tokens)

def load_and_preprocess(file_path):
    # Load the data
    df = pd.read_csv(file_path, header=None, names=['URL', 'Summary'])
    # Preprocess the text
    df['processed_summary'] = df['Summary'].apply(preprocess_text)
    return df

def vectorize_text(text_data):
    vectorizer = TfidfVectorizer()
    tfidf_matrix = vectorizer.fit_transform(text_data)
    return tfidf_matrix, vectorizer

def find_closest_summary(user_query, df, tfidf_matrix, vectorizer):
    processed_query = preprocess_text(user_query)
    query_vector = vectorizer.transform([processed_query])
    similarities = cosine_similarity(query_vector, tfidf_matrix)
    closest_idx = np.argmax(similarities)
    return df['URL'].iloc[closest_idx]


file_path = 'wiki.csv'
df = load_and_preprocess(file_path)
tfidf_matrix, vectorizer = vectorize_text(df['processed_summary'])

while True:
    user_query = input("Enter your query: ")
    closest_url = find_closest_summary(user_query, df, tfidf_matrix, vectorizer)
    print(f"The URL of the closest summary is: {closest_url}")
